module Grants
  class Manager

      def self.grants
        {
          reset_password: {
              name: 'Passwort zurücksetzen',
              scopes: {
                  system_wide: {
                      name: 'Systemweit',
                      detailed_description: "keine Einschränkungen" ,
                      user_types: [::Manager],
                      default_for: [],
                      manager: -> ability,manager do
                        ability.can :reset_password, ::Manager
                      end
                  },
                  own_institutions: {
                      name: 'in eigenen Einrichtungen',
                      detailed_description: '
                      der Verwalter darf Verwaltern unterhalb seiner eigenen Einrichtung und in seiner eigenen Einrichtung das Passwort zurücksetzen.
                      ',
                      user_types: [::Manager],
                      default_for:  [],
                      manager: -> ability, manager do
                        ability.can :reset_password, ::Manager, {
                            institution:{
                                all_parent_institutions: { parent_id: manager.institution_id }
                            }
                        }
                        ability.can :reset_password, ::Manager, {
                            institution_id: manager.institution_id
                        }
                      end,
                  },
              }
          },
          create: {
              name: 'Anlegen',
              scopes: {
                  system_wide: {
                      name: 'Systemweit',
                      detailed_description: "keine Einschränkungen" ,
                      user_types: [::Manager],
                      default_for: [],
                      manager: -> ability,manager do
                        ability.can :create, ::Manager
                      end
                  },
                  own_institutions: {
                      name: 'in eigenen Einrichtungen',
                      detailed_description: '
                      der Verwalter darf Verwalter unterhalb seiner eigenen Einrichtung und in seiner eigenen Einrichtung anlegen.
                      Beim Versuch einen Verwalter anzulegen dürfen nirgendwo andere Einrichtungen angezeigt werden.
                      ',
                      user_types: [::Manager],
                      default_for:  [::Manager],
                      manager: -> ability, manager do
                        ability.can :create, ::Manager, {
                            institution:{
                                all_parent_institutions: { parent_id: manager.institution_id }
                            }
                        }
                        ability.can :create, ::Manager, {
                            institution_id: manager.institution_id
                        }
                      end,
                  },
              }
          },
          read: {
              name: 'Anzeigen',
              scopes: {
                  system_wide: {
                      name: 'Systemweit',
                      detailed_description: "keine Einschränkungen" ,
                      user_types: [::Manager],
                      default_for: [],
                      manager: -> ability do
                        ability.can :read, ::Manager
                      end
                  },
                  own_institutions: {
                      name: 'in eigenen Einrichtungen',
                      detailed_description: '
                      ein Verwalter darf Verwalter unterhalb seiner eigenen Einrichtung und in seiner eigenen Einrichtung sehen.
                      ein Lehrer darf Verwalter aus Einrichtungen sehen, die genau eine Ebene über einer Einrichtung, in der er arbeitet, liegen.
                      Es dürfen keine Verwalter aus anderen Einrichtungen angezeigt werden
                      ',
                      user_types: [::Manager,::Teacher],
                      default_for:  [::Manager,::Teacher],
                      manager: -> ability, manager do
                        ability.can :read, ::Manager, {
                            institution: {
                               all_parent_institutions: { parent_id: manager.institution_id },
                                institution_type: ::Institution::TYPE_ORGANISATIONAL
                            }
                        }
                        ability.can :read, ::Manager, {
                            institution_id: manager.institution_id
                        }
                      end,
                      teacher: -> ability, teacher do
                        ability.can :read, ::Manager, {
                            institution:{
                                child_institutions: {
                                    teacher_institution_relations: {
                                        teacher_id: teacher.id
                                    },
                                },
                                institution_type: ::Institution::TYPE_ORGANISATIONAL
                            }
                        }
                      end
                  },
              }
          },
          update: {
              name: 'Bearbeiten',
              scopes: {
                  system_wide: {
                      name: 'Systemweit',
                      detailed_description: "keine Einschränkungen" ,
                      user_types: [::Manager],
                      default_for: [],
                      manager: -> ability do
                        ability.can :update, ::Manager
                      end
                  },
                  own_institutions: {
                      name: 'in eigenen Einrichtungen',
                      detailed_description: '
                      ein Verwalter darf Verwalter unterhalb seiner eigenen Einrichtung und in seiner eigenen Einrichtung bearbeiten.
                      Es dürfen keine Bearbeiten-Buttons für Verwalter aus anderen Einrichtungen angezeigt werden
                      ',
                      user_types: [::Manager],
                      default_for:  [::Manager],
                      manager: -> ability, manager do
                        ability.can :update, ::Manager, {
                            institution:{
                              all_parent_institutions: { parent_id: manager.institution_id }
                            }
                        }
                        ability.can :update, ::Manager, {
                            institution_id: manager.institution_id
                        }
                      end
                  },
              }
          },
          update_roles: {
              name: 'Rollen bearbeiten',
              scopes: {
                  system_wide: {
                      name: 'Systemweit',
                      user_types: [::Manager],
                      default_for: [],
                      manager: -> ability do
                        ability.can :update_roles, ::Manager
                      end

                  },
                  own_institutions: {
                      name: 'in eigenen Einrichtungen',
                      user_types: [::Manager],
                      default_for:  [::Manager],
                      manager: -> ability, manager do
                        ability.can :update_roles, ::Manager, {
                            institution: {
                                all_parent_institutions: { parent_id: manager.institution_id },
                                institution_type: ::Institution::TYPE_ORGANISATIONAL
                            }
                        }
                        ability.can :update_roles, ::Manager, {
                            institution_id: manager.institution_id
                        }
                      end
                  }
              }
          },
          destroy: {
              name: 'Löschen',
              scopes: {
                  system_wide: {
                      name: 'Systemweit',
                      detailed_description: "keine Einschränkungen" ,
                      user_types: [::Manager],
                      default_for: [],
                      manager: -> ability do
                        ability.can :destroy, ::Manager
                      end
                  },
                  own_institutions: {
                      name: 'in eigenen Einrichtungen',
                      detailed_description: '
                      ein Verwalter darf Verwalter unterhalb seiner eigenen Einrichtung und in seiner eigenen Einrichtung löschen.
                      Es dürfen keine Löschen-Buttons für Verwalter aus anderen Einrichtungen angezeigt werden
                      ',
                      user_types: [::Manager],
                      default_for:  [],
                      manager: -> ability, manager do
                        ability.can :destroy, ::Manager, {
                            institution:{
                                all_parent_institutions: { parent_id: manager.institution_id }
                            }
                        }
                        ability.can :destroy, ::Manager, {
                            institution_id: manager.institution_id
                        }
                      end
                  },
              }
          }
        }
      end
  end
end