module Grants
  class Teacher
      def self.grants
        {
            reset_password: {
                name: 'Passwort zurücksetzen',
                scopes: {
                    own_institutions: {
                        name: 'in eigenen Einrichtungen',
                        user_types: [::Manager],
                        default_for:  [],
                        manager: -> ability, manager do
                          ability.can :reset_password, ::Teacher, {
                              teacher_institution_relations: {
                                  institution:{
                                      all_parent_institutions: { parent_id: manager.institution_id }
                                  }
                              }
                          }
                        end,
                    },

                    leaded_institutions: {
                        name: 'in administrierten Einrichtungen (Schuladmin)',
                        user_types: [::Teacher],
                        default_for:  [::Teacher],
                        teacher: -> ability, teacher do
                          ability.can :reset_password, ::Teacher, {
                              teacher_institution_relations: {
                                  institution:{
                                      teacher_institution_relations: {
                                          is_school_admin: true,
                                          teacher_id: teacher.id
                                      }
                                  }
                              }
                          }
                          t_ids = ::Teacher.connection.select_values("
                        select institutions.id from institutions
                        join teacher_institution_relations as t2 on institutions.id = t2.institution_id
                        where t2.teacher_id=#{teacher.id} and t2.is_school_admin=true").map(&:to_i)
                          ability.can :create_teacher, ::Institution, { id: t_ids  }
                        end
                    }
                }
            },
            create: {
              name: 'Anlegen',
              scopes: {
                  system_wide: {
                      name: 'Systemweit',
                      user_types: [::Manager],
                      default_for: [],
                      manager: -> ability do
                        ability.can :create, ::Teacher
                        ability.can :create_teacher, ::Institution
                      end
                  },
                  own_institutions: {
                      name: 'in eigenen Einrichtungen',
                      user_types: [::Manager,::Teacher],
                      default_for:  [::Manager],
                      manager: -> ability, manager do
                        ability.can :create, ::Teacher, {
                            teacher_institution_relations: {
                                institution:{
                                    all_parent_institutions: { parent_id: manager.institution_id }
                                }
                            }
                        }
                        ability.can :create_teacher, ::Institution, {
                            all_parent_institutions: { parent_id: manager.institution_id }
                        }
                      end,
                      teacher: -> ability, teacher do
                        ability.can :create, ::Teacher, {
                            teacher_institution_relations: {
                              institution:{
                                  teacher_institution_relations: {
                                      teacher_id: teacher.id
                                  }
                              }
                            }
                        }
                        t_ids = ::Teacher.connection.select_values("
                        select institutions.id from institutions
                        join teacher_institution_relations as t2 on institutions.id = t2.institution_id
                        where t2.teacher_id=#{teacher.id}").map(&:to_i)
                        ability.can :create_teacher, ::Institution, { id: t_ids  }
                      end
                  },
                  leaded_institutions: {
                      name: 'in administrierten Einrichtungen (Schuladmin)',
                      user_types: [::Teacher],
                      default_for:  [::Teacher],
                      teacher: -> ability, teacher do
                        ability.can :create, ::Teacher, {
                            teacher_institution_relations: {
                              institution:{
                                  teacher_institution_relations: {
                                      is_school_admin: true,
                                      teacher_id: teacher.id
                                  }
                              }
                            }
                        }
                        t_ids = ::Teacher.connection.select_values("
                        select institutions.id from institutions
                        join teacher_institution_relations as t2 on institutions.id = t2.institution_id
                        where t2.teacher_id=#{teacher.id} and t2.is_school_admin=true").map(&:to_i)
                        ability.can :create_teacher, ::Institution, { id: t_ids  }
                      end
                  }
              }
          },
          read: {
              name: 'Anzeigen',
              scopes: {
                  system_wide: {
                      name: 'Systemweit',
                      user_types: [::Manager,::Teacher],
                      default_for: [],
                      manager: -> ability do
                        ability.can :read, ::Teacher
                      end,
                      teacher: -> ability do
                        ability.can :read, ::Teacher
                      end
                  },
                  own_institutions: {
                      name: 'in eigenen Einrichtungen',
                      user_types: [::Manager,::Teacher],
                      default_for:  [::Manager,::Teacher],
                      manager: -> ability, manager do
                        ability.can :read, ::Teacher, {
                            teacher_institution_relations: {
                                institution: {
                                    all_parent_institutions: { parent_id: manager.institution_id }
                                }
                            }
                        }
                      end,
                      teacher: -> ability, teacher do
                        t_ids = ::Teacher.connection.select_values("
                        select teachers.id from teachers
                        join teacher_institution_relations on teachers.id = teacher_institution_relations.teacher_id
                        join institutions on institutions.id = teacher_institution_relations.institution_id
                        join teacher_institution_relations as t2 on institutions.id = t2.institution_id
                        where t2.teacher_id=#{teacher.id}").map(&:to_i)
                        ability.can :read, ::Teacher, { id: t_ids  }
                      end
                  },
              }
          },
          update: {
              name: 'Bearbeiten',
              scopes: {
                  system_wide: {
                      name: 'Systemweit',
                      user_types: [::Manager,::Teacher],
                      default_for: [],
                      manager: -> ability do
                        ability.can :update, ::Teacher
                      end,
                      teacher: -> ability do
                        ability.can :update, ::Teacher
                      end

                  },
                  own: {
                      name: 'Eigene Daten',
                      user_types: [::Manager,::Teacher],
                      default_for:  [::Manager,::Teacher],
                      manager: -> ability, manager do
                        ability.can :update, ::Manager, { id: manager.id }
                      end,
                      teacher: -> ability, teacher do
                        ability.can :update, ::Teacher, { id: teacher.id }
                      end

                  },
                  own_institutions: {
                      name: 'in eigenen Einrichtungen',
                      user_types: [::Manager,::Teacher],
                      default_for:  [::Manager],
                      manager: -> ability, manager do
                        ability.can :update, ::Teacher, {
                            teacher_institution_relations: {
                                institution:{
                                    all_parent_institutions: { parent_id: manager.institution_id }
                                }
                            }
                        }
                      end,
                      teacher: -> ability, teacher do
                        t_ids = ::Teacher.connection.select_values("
                        select teachers.id from teachers
                        join teacher_institution_relations on teachers.id = teacher_institution_relations.teacher_id
                        join institutions on institutions.id = teacher_institution_relations.institution_id
                        join teacher_institution_relations as t2 on institutions.id = t2.institution_id
                        where t2.teacher_id=#{teacher.id}").map(&:to_i)
                        ability.can :update, ::Teacher, { id: t_ids  }
                      end
                  },
                  leaded_institutions: {
                      name: 'in administrierten Einrichtungen (Schuladmin)',
                      user_types: [::Teacher],
                      default_for:  [::Teacher],
                      teacher: -> ability, teacher do
                        t_ids = ::Teacher.connection.select_values("
                        select teachers.id from teachers
                        join teacher_institution_relations on teachers.id = teacher_institution_relations.teacher_id
                        join institutions on institutions.id = teacher_institution_relations.institution_id
                        join teacher_institution_relations as t2 on institutions.id = t2.institution_id
                        where t2.teacher_id=#{teacher.id} and t2.is_school_admin=true").map(&:to_i)
                        ability.can :update, ::Teacher, { id: t_ids  }
                      end
                  }
              }
          },
          update_roles: {
              name: 'Rollen bearbeiten',
              scopes: {
                  system_wide: {
                      name: 'Systemweit',
                      user_types: [::Manager],
                      default_for: [],
                      manager: -> ability do
                        ability.can :update_roles, ::Teacher
                      end

                  },
                  own_institutions: {
                      name: 'in eigenen Einrichtungen',
                      user_types: [::Manager,::Teacher],
                      default_for:  [::Manager],
                      manager: -> ability, manager do
                        ability.can :update_roles, ::Teacher, {
                            teacher_institution_relations: {
                                institution:{
                                    all_parent_institutions: { parent_id: manager.institution_id }
                                }
                            }
                        }
                      end,
                      teacher: -> ability, teacher do
                        t_ids = ::Teacher.connection.select_values("
                        select teachers.id from teachers
                        join teacher_institution_relations on teachers.id = teacher_institution_relations.teacher_id
                        join institutions on institutions.id = teacher_institution_relations.institution_id
                        join teacher_institution_relations as t2 on institutions.id = t2.institution_id
                        where t2.teacher_id=#{teacher.id}").map(&:to_i)
                        ability.can :update_roles, ::Teacher, { id: t_ids  }
                      end
                  },
                  leaded_institutions: {
                      name: 'in administrierten Einrichtungen (Schuladmin)',
                      user_types: [::Teacher],
                      default_for:  [::Teacher],
                      teacher: -> ability, teacher do
                        t_ids = ::Teacher.connection.select_values("
                        select teachers.id from teachers
                        join teacher_institution_relations on teachers.id = teacher_institution_relations.teacher_id
                        join institutions on institutions.id = teacher_institution_relations.institution_id
                        join teacher_institution_relations as t2 on institutions.id = t2.institution_id
                        where t2.teacher_id=#{teacher.id} and t2.is_school_admin=true").map(&:to_i)
                        ability.can :update_roles, ::Teacher, { id: t_ids  }
                      end
                  }
              }
          },
          destroy: {
              name: 'Löschen',
              scopes: {
                  system_wide: {
                      name: 'Systemweit',
                      user_types: [::Manager,::Teacher],
                      default_for: [],
                      manager: -> ability do
                        ability.can :destroy, ::Teacher
                      end,
                      teacher: -> ability do
                        ability.can :destroy, ::Teacher
                      end
                  },
                  own_institutions: {
                      name: 'in eigenen Einrichtungen',
                      user_types: [::Manager,::Teacher],
                      default_for:  [::Manager],
                      manager: -> ability, manager do
                        ability.can :destroy, ::Teacher, {
                            teacher_institution_relations: {
                                institution:{
                                    all_parent_institutions: { parent_id: manager.institution_id }
                                }
                            }
                        }
                      end,
                      teacher: -> ability, teacher do
                        t_ids = ::Teacher.connection.select_values("
                        select teachers.id from teachers
                        join teacher_institution_relations on teachers.id = teacher_institution_relations.teacher_id
                        join institutions on institutions.id = teacher_institution_relations.institution_id
                        join teacher_institution_relations as t2 on institutions.id = t2.institution_id
                        where t2.teacher_id=#{teacher.id}").map(&:to_i)
                        ability.can :destroy, ::Teacher, { id: t_ids  }
                      end
                  },
                  leaded_institutions: {
                      name: 'in administrierten Einrichtungen (Schuladmin)',
                      user_types: [::Teacher],
                      default_for:  [::Teacher],
                      teacher: -> ability, teacher do
                        t_ids = ::Teacher.connection.select_values("
                        select teachers.id from teachers
                        join teacher_institution_relations on teachers.id = teacher_institution_relations.teacher_id
                        join institutions on institutions.id = teacher_institution_relations.institution_id
                        join teacher_institution_relations as t2 on institutions.id = t2.institution_id
                        where t2.teacher_id=#{teacher.id} and t2.is_school_admin=true").map(&:to_i)
                        ability.can :destroy, ::Teacher, { id: t_ids  }

                      end
                  }
              }
          }
        }
      end
  end
end