class CareerItemsClientGroupController < ApplicationController

  before_action :set_client, only: [:index, :new, :create]

  def index
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    add_breadcrumb "#{view_context.show_person(@client)}", dashboard_client_path(@client), title: "Übersicht anzeigen"
    add_breadcrumb "Laufbahn", client_career_items_path(@client), title: "gesamte Laufbahn anzeigen"
    add_breadcrumb "#{CareerItem::CHANGE_TYPES[0]}"

    @career_items_grid = CareerItemsClientGroupGrid.new(params[:career_items_client_group_grid]) do |scope|
      scope.where(client_id: params[:client_id]).page(params[:page]).per(25)
    end
  end

  def new
    authorize!(:update, @client)
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    add_breadcrumb "#{view_context.show_person(@client)}", dashboard_client_path(@client), title: "Übersicht anzeigen"
    add_breadcrumb "#{CareerItem::CHANGE_TYPES[0]}"

    @career_item = @client.career_items.build({change_type: CareerItem::CHANGE_TYPE_CLIENT_GROUP})

    # das aktuelle Schuljahr
    @current_semester = SystemSetting.get_instance.semester

    # die im Dropdown verfügbaren Klassen
    @available_client_groups = {}
    # alle an der aktuellen Einrichtung des Schülers verfügbaren Klassen im aktuellen Semester
    # gruppiert nach Einrichtung ??
    # mit ergänzender Info zum Schuljahr
    @available_client_groups =
      @client.institution.client_groups.where(semester: @current_semester).order(semester_id: :desc, name: :asc)
        .map{ |cg| [
          cg == @client.client_group ? cg.name + " (aktuell)" : cg.name,
          #cg.id, {'data-subtext': cg.semester.try(:name)}] }
          cg.id, {}] } << ['keine Klasse',nil,{}]
  end

  def create
    @client.assign_attributes(client_params)
    authorize!(:update, @client)
    respond_to do |format|
      if @client.save
        success = "Die neue Klassenzugehörigkeit wurde erfolgreich gespeichert. Die Laufbahn von " + view_context.show_person(@client) + " wurde entsprechend aktualisiert."
        format.html { redirect_to dashboard_client_path(@client), success: success }
      else
        format.html { render :new }
      end
    end
  end


  private
    # Use callbacks to share common setup or constraints between actions.

    def client_params
      params.require(:client).permit(:client_group_id, :is_former_client, career_items_attributes: [:change_type, :description])
    end

    def set_client
      @client = Client.find(params[:client_id])
    end

end
