class ClientsController < ApplicationController
  include PdfConcern


  before_action :set_client, only: [:edit, :update, :destroy, :dashboard, :raise_level, :lower_level,
                                    :edit_former_career,:update_former_career,:edit_career,:update_career,
                                    :edit_expertise, :update_expertise]

  # GET /clients/1/dashboard
  def dashboard
    authorize!(:read,@client)
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    add_breadcrumb "#{view_context.show_person(@client)}", dashboard_client_path(@client), title: "Übersicht anzeigen"
    # in fact, these are ALL portfolio items. There is no additional view for client portfolio items.
    @latest_portfolio_items = @client.portfolio_items.order(start: :desc, created_at: :desc).includes([:portfolio_specialization,:portfolio_step])
    @career_items = @client.career_items.order(effective_date: :desc,created_at: :desc).limit(500)
    #@notifications = @client.system_notifications
    @available_supporters = build_available_supporters
    respond_to do |format|
      format.html
      format.pdf {
        render_pdf(
          "Übersicht-#{@client.first_name}-#{@client.last_name}.pdf",
          :dashboard,
          title: "Übersicht_#{@client.first_name}_#{@client.last_name}",
          header_left: "#{@client.first_name} #{@client.last_name}"
        )
      }
    end
  end

  def select_portfolio_item_date
    @client = Client.where( id: params[:client_id]).take!
    if @client.blank?
      flash[:danger] = 'Konnte den Schüler nicht finden'
      redirect_to :back and return
    end
    @portfolio_item = ::Portfolio::Item.new({portfolio: @client.running_portfolio})
    authorize!(:create, ::Portfolio::Item)
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    add_breadcrumb "#{view_context.show_person(@client)}", dashboard_client_path(@client), title: "Übersicht anzeigen"
    add_breadcrumb "neuer Eintrag im Portfolio", select_portfolio_item_date_clients_path()
  end

  def select_portfolio_item_step
    @client = Client.where( id: params[:client][:id]).take!
    if @client.blank?
      flash[:danger] = 'Konnte den Schüler nicht finden'
      redirect_to :back and return
    end
    @portfolio_item = ::Portfolio::Item.new({portfolio: @client.running_portfolio}.merge(params.require(:portfolio_item).permit( :start)))
    ::Portfolio::Item.validators_on(:start).each { |validator| validator.validate_each(@portfolio_item, :start, @portfolio_item.start) }
    if @portfolio_item.errors[:start].any?
      flash[:danger] = @portfolio_item.errors[:start].join(', ')
      redirect_to :back and return
    end
    authorize!(:create, ::Portfolio::Item)
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    add_breadcrumb "#{view_context.show_person(@client)}", dashboard_client_path(@client), title: "Übersicht anzeigen"
    add_breadcrumb "neuer Eintrag im Portfolio", select_portfolio_item_step_clients_path()
  end

  def select_new_client_institution
    unless ClientGroup.any?
      flash[:info] = "Es sind noch keine Klassen vorhanden. Bitte anlegen."
      return redirect_to :new_client_group
    end
    @client = Client.new
    authorize!(:create, Client)
  end

  def edit_next_semester_client_groups
    if Semester.following.first.blank?
      flash[:danger] = 'Konnte das nächste Schuljahr nicht finden'
      redirect_to :back and return
    end
    authorize!(:update, Client)
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    add_breadcrumb "Klassen im Schuljahr #{Semester.following.take!.name}", :edit_next_semester_client_groups_clients_path
    # SystemSetting.get_instance.load_updated_semesters this is done in cron task
    @clients = Client.accessible_by(current_ability,:update).order(:last_name,:first_name)
  end

  def update_next_semester_client_groups
    if Semester.following.take!.blank?
      flash[:danger] = 'Konnte das nächste Schuljahr nicht finden'
      redirect_to :back and return
    end
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    add_breadcrumb "Klassen im Schuljahr #{Semester.following.take!.name}", :edit_next_semester_client_groups_clients_path
    _hsh = params.require(:clients)
    @clients = []
    errors = false
    _hsh.each do |params|
      next unless client = Client.where(id: params[1]["id"]).take
      client.next_semester_client_group_id = params[1][ "next_semester_client_group_id" ]
      client.is_former_client = params[1][ "is_former_client" ]
      client.save
      errors ||= client.errors.any?
      @clients << client
    end
    respond_to do |format|
      unless errors
        format.html { redirect_to edit_next_semester_client_groups_clients_path, {success: "Gespeichert"} }
      else
        format.html { render :edit_next_semester_client_groups, {warning: "Einige Klassen konnten nicht gespeichert werden"} }
      end
    end
  end

  def index
    authorize!(:read, Client )
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    @client_grid = ClientGrid.new(params[:client_grid]) do |scope|
      scope.accessible_by(current_ability).page(params[:page]).per(25)
    end
  end


  def new
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    add_breadcrumb "neu", :new_client_path
    @client = Client.new(params.require(:client).permit(:institution_id))
    unless @client.institution.present?
      return redirect_to :select_new_client_institution
    end
    authorize!(:create, Client)
    @available_institutions = build_available_institutions
    @available_denominations = build_available_denominations
    @available_languages = build_available_languages
    @available_nationalities = build_available_nationalities
    @available_semesters = build_available_semesters
  end

  def new_from_client_duplicate_request
    @client_duplicate_request = ClientDuplicateRequest.where(id: params[:id]).take!
    add_breadcrumb "Duplikatsauflösung", client_duplicate_request_path(@client_duplicate_request)
    add_breadcrumb "Schüler anlegen", new_from_client_duplicate_request_client_path(@client_duplicate_request)
    @client = Client.new(@client_duplicate_request.request)
    @client.client_duplicate_request = @client_duplicate_request
    authorize!(:create, Client)
    @available_institutions = build_available_institutions
    @available_denominations = build_available_denominations
    @available_languages = build_available_languages
    @available_nationalities = build_available_nationalities
    @available_semesters = build_available_semesters
    render :new
  end

  def edit
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    add_breadcrumb "#{view_context.show_person(@client)}", dashboard_client_path(@client), title: "Übersicht anzeigen"
    add_breadcrumb "bearbeiten", edit_client_path(@client)
    @available_institutions = build_available_institutions
    @available_denominations = build_available_denominations
    @available_languages = build_available_languages
    @available_nationalities = build_available_nationalities
    @available_semesters = build_available_semesters
    authorize!(:update, @client)
  end

  def edit_former_career
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    add_breadcrumb "#{view_context.show_person(@client)}", dashboard_client_path(@client), title: "Übersicht anzeigen"
    add_breadcrumb "vorherige Laufbahn bearbeiten", edit_former_career_client_path(@client)
    authorize!(:update, @client)
  end

  def edit_career
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    add_breadcrumb "#{view_context.show_person(@client)}", dashboard_client_path(@client), title: "Übersicht anzeigen"
    add_breadcrumb "Laufbahn bearbeiten", edit_career_client_path(@client)
    authorize!(:update, @client)
  end

  def edit_expertise
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    add_breadcrumb "#{view_context.show_person(@client)}", dashboard_client_path(@client), title: "Übersicht anzeigen"
    add_breadcrumb "Gutachten bearbeiten", edit_expertise_client_path(@client)
    authorize!(:read, @client)
  end

  def update_expertise
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    add_breadcrumb "#{view_context.show_person(@client)}", dashboard_client_path(@client), title: "Übersicht anzeigen"
    add_breadcrumb "Gutachten bearbeiten", edit_expertise_client_path(@client)
    @client.assign_attributes(client_expertise_params)
    authorize!(:read, @client)
    respond_to do |format|
      if @client.save
        format.html { redirect_to dashboard_client_url(@client), {
            success: "Die Daten von " + view_context.show_person(@client) + " wurden erfolgreich aktualisiert." } }
      else
        error_message = 'Bitte prüfen Sie Ihre Eingaben.'
        @client.errors[:base].each do |message|
          error_message += '<br/>' + message
        end
        flash.now[:danger] = error_message.html_safe
        format.html { render :edit_expertise }
      end
    end
  end

  def update_former_career
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    add_breadcrumb "#{view_context.show_person(@client)}", dashboard_client_path(@client), title: "Übersicht anzeigen"
    add_breadcrumb "vorherige Laufbahn bearbeiten", edit_former_career_client_path(@client)
    @client.assign_attributes(client_former_career_params)
    authorize!(:update, @client)
    respond_to do |format|
      if @client.save
        format.html { redirect_to dashboard_client_url(@client), {
            success: "Die Daten von " + view_context.show_person(@client) + " wurden erfolgreich aktualisiert." } }
      else
        error_message = 'Bitte prüfen Sie Ihre Eingaben.'
        @client.errors[:base].each do |message|
          error_message += '<br/>' + message
        end
        flash.now[:danger] = error_message.html_safe
        format.html { render :edit_former_career }
      end
    end
  end

  def update_career
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    add_breadcrumb "#{view_context.show_person(@client)}", dashboard_client_path(@client), title: "Übersicht anzeigen"
    add_breadcrumb " Laufbahn bearbeiten", edit_career_client_path(@client)
    @client.assign_attributes(client_system_career_params)
    authorize!(:update, @client)
    respond_to do |format|
      if @client.save
        format.html { redirect_to dashboard_client_url(@client), {
            success: "Die Daten von " + view_context.show_person(@client) + " wurden erfolgreich aktualisiert." } }
      else
        error_message = 'Bitte prüfen Sie Ihre Eingaben.'
        @client.errors[:base].each do |message|
          error_message += '<br/>' + message
        end
        flash.now[:danger] = error_message.html_safe
        format.html { render :edit_career }
      end
    end
  end

  def create
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    add_breadcrumb "neu", :new_client_path
    @client = Client.new(client_params)
    authorize!(:create, @client)
    @available_institutions = build_available_institutions
    @available_denominations = build_available_denominations
    @available_languages = build_available_languages
    @available_nationalities = build_available_nationalities
    @available_semesters = build_available_semesters
    respond_to do |format|
      persisted = @client.save
      success = view_context.show_person(@client) + "wurde eingetragen"
      if persisted
        format.html { redirect_to user_dashboard_path, {success: success, warning: warning} }
      else
        error_message = 'Bitte prüfen Sie Ihre Eingaben.'
        @client.errors[:base].each do |message|
          error_message += '<br/>' + message
        end
        flash.now[:danger] = error_message.html_safe
        format.html { render :new }
      end
    end
  end

  def check_duplicates
    @first_name = params['first_name']
    @last_name = params['last_name']
    @bday = params['bday']
    @id = params['id']
    @duplicates = Client.remote_duplicates( @first_name, @last_name, @bday, @id )
    respond_to do |format|
      format.html { render :check_duplicates, layout: false }
    end
  end

  def update
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    add_breadcrumb "#{view_context.show_person(@client)}", dashboard_client_path(@client), title: "Übersicht anzeigen"
    add_breadcrumb "bearbeiten", edit_client_path(@client)
    @available_institutions = build_available_institutions
    @available_denominations = build_available_denominations
    @available_languages = build_available_languages
    @available_nationalities = build_available_nationalities
    @available_semesters = build_available_semesters
    @client.assign_attributes(client_params)
    authorize!(:update, @client)
    respond_to do |format|
      if @client.save
        format.html { redirect_to dashboard_client_url(@client), {
          success: "Die Daten von " + view_context.show_person(@client) + " wurden erfolgreich aktualisiert.",
          warning: warning} }
      else
        error_message = 'Bitte prüfen Sie Ihre Eingaben.'
        @client.errors[:base].each do |message|
          error_message += '<br/>' + message
        end
        flash.now[:danger] = error_message.html_safe
        format.html { render :edit }
      end
    end
  end

  def destroy
    authorize! :destroy, @client
    respond_to do |format|
      if @client.destroy
        format.html { redirect_to clients_url, success: 'Klient erfolgreich gelöscht.' }
      else
        format.html { redirect_to clients_url, danger: 'Klient konnte nicht gelöscht werden. ' + @client.errors[:base].first }
      end
    end
  rescue ActiveRecord::RecordNotDestroyed => invalid
    respond_to do |format|
      format.html { redirect_to clients_url, danger: 'Klient konnte nicht gelöscht werden. ' }
    end
  end

  def raise_level
    if @client.has_closed_expertise?
      next_level = 3
    else
      next_level = @client.running_portfolio.next_rule_level
    end
    @client.raise_portfolio_level!
    respond_to do |format|
      format.html { redirect_to dashboard_client_url(@client), { success: "Die Förderstufe von #{@client.first_name} #{@client.last_name} wurde erhöht auf Stufe #{next_level}. "} }
    end
  end

  def lower_level
    previous_level = @client.running_portfolio.last_rule_level
    @client.lower_portfolio_level!
    respond_to do |format|
      format.html { redirect_to dashboard_client_url(@client), { success: "Die Förderstufe von #{@client.first_name} #{@client.last_name} wurde zurückgesetzt auf Stufe #{previous_level}. "} }
    end
  end

  private
    # Use callbacks to share common setup or constraints between actions.
    def set_client
      @client = Client.where(id: params[:id]).take
    end

    def client_former_career_params
      params.require(:client).permit(
          :id, former_career_items_attributes: [
              :id, :description, :start_at, :change_type, :teacher_name,:institution_name,:client_group_name, :_destroy
          ]
      )
    end

    def client_expertise_params
      params.require(:client).permit(
          :id, :expertise_manager_id, :expertise_institution_id, :expertise_teacher_id, :expertise_school_inspector_id,
          :expertise_manager_result,:expertise_inspector_result,:expertise_institution_result,:expertise_teacher_result,
          :expertise_manager_result_text,:expertise_inspector_result_text,:expertise_institution_result_text,:expertise_teacher_result_text
      )
    end

    def client_system_career_params
      params.require(:client).permit(
          :id, system_career_items_attributes: [
          :id, :effective_date
      ]
      )
    end

    def client_params
      params.require(:client).permit(
        :first_name, :last_name, :gender, :description, :bday, :birth_place,:dont_check_similar_clients,
        :nationality, :language, :denomination, :initial_special_ed_necessary,
        :school_attendance_since, :school_attendance_since_article, :school_attendance_postponed, :institution_id, :client_group_id,
        :client_duplicate_request_id,
        address_attributes: [
            :id,:street,:city,:region,:country,:number,:zip
        ],
        client_guardian_relations_attributes: [
          :id, :has_custody,:guardian_id, :category,:save_anyway,
          :_destroy,
          guardian_attributes: [
            :id, :first_name, :last_name, :gender, :phone, :mobile, :email, :_destroy,
            address_attributes: [
                :id,:street,:city,:region,:country,:number,:zip
            ]
          ]
        ]
      )
    end



    def build_available_denominations
      CsbApi::Denomination.all.map{|d| [d.name, d.name]}
    end

    def build_available_institutions
      if current_user.is_teacher?
        current_user.identifiable.institutions
      else
        Institution.education_facilities
      end
    end

    def build_available_languages
      CsbApi::Language.all.map{|l| [l.name, l.name]}
    end

    def build_available_nationalities
      CsbApi::Nationality.all.map{|n| [n.name, n.name]}
    end

    def build_available_semesters
      Semester.following_and_current
    end

    def build_available_supporters
      supporters = {}
      supporters["nicht aufgeführt?"] = [["neue externe Begleitperson eintragen", 'new']]
      Supporter.all.each do |supporter|
        option = [view_context.show_supporter(supporter), supporter.id]
        key = (institution = supporter.institution) ? institution.name : "ohne Einrichtung"
        if supporters.has_key?(key)
          supporters[key] << option
        else
          supporters[key] = [option]
        end
      end
      supporters
    end

end