module PdfConcern
  extend ActiveSupport::Concern

  # send a pdf to the browser, named <filename>, rendered view tempate <template>.
  # options are more or less directly passed to wkhtmltopdf
  # have a look at http://wkhtmltopdf.org/usage/wkhtmltopdf.txt
  # warning: wkhtmltopdf is very sensitive about the order of parameters.
  # correction: sensitive is not the right expression, its a nightmare.
  # ALL used parameters must be listed in the parameters hash and it should be tested if they
  # trigger any errors at their position.
  def render_pdf(filename,template,options={})
    parameters = {
        page_size: 'A4',
        title: "",
        header_left: "",
        header_right: "",
        footer_center: "Seite [page]/[toPage]",
        footer_spacing: 5, # in mm
        header_spacing: 5, # in mm
        footer_line: true,
        header_line: true,
        toc:false,
        toc_header_text: 'Inhaltsverzeichnis',
        toc_text_size_shrink: 0.95,
        disable_dotted_lines: true, # for toc
        print_media_type: true,
        cover: nil
    }
    # wkhtmltopdf can read the cover only from a file, so we must create a tempfile
    parameters.merge!(options)
    if parameters[:toc] == false
      parameters.delete(:toc)
      parameters.delete(:toc_header_text)
      parameters.delete(:toc_text_size_shrink)
      parameters.delete(:disable_dotted_lines)
    end
    if options[:cover]
      tempfile = Tempfile.new([ "cover", '.html'] )
      tempfile.write(render_to_string(options[:cover], formats: ['pdf']))
      tempfile.close
      parameters[:cover] = tempfile.path
    else
      parameters.delete(:cover)
    end
    send_data(PDFKit.new(render_to_string(template,formats: ['pdf']), parameters).to_pdf,
              type: 'application/pdf', disposition: 'inline',
              filename: filename)
  ensure
    tempfile.unlink  if parameters[:cover]
  end

end