class ClientGrid < BaseGrid

  scope do
    Client.all
  end


  # This maps given id of client group on readable names
  def filter_value_as_string(name)
    value = super(name)
    name= name.name if name.respond_to? :name # weird thing, seems to be only symbol sometimes, then Filter class other time
    case name
      when :client_group
        value = ClientGroup.where(id: value).pluck(:name).first if /^\d+$/.match(value)
    end
    return value
  end

  filter(:id, :string, header: "ID")
  filter(:is_former_client, :xboolean, header: "Ehemalige", default: Datagrid::Filters::BooleanEnumFilter::NO)  do |value|
    if value == true
      self.where(["clients.is_former_client = ? AND clients.client_group_id IS NULL",  value])
    else
      self.where(["clients.client_group_id IS NOT NULL",  value])
    end
  end
  filter(:client_group, :string, header: 'Klasse') do |value,scope,grid|
    if /^\d+$/.match(value)
      scope.where(['clients.client_group_id=? ',value])
    else
      scope.where(['clients.client_group_id IN (SELECT id FROM client_groups WHERE name ILIKE ? ) ', "%#{value}%"])
    end
  end

  filter(:institutions, :enum, header: "Einrichtung",  :select => proc { Institution.education_facilities.map {|c| [c.name, c.id] }}) do |value|
    self.where('clients.institution_id = ?',value)
  end

  # exakte suche nach vorname + name bzw. nur name, der Benutzer soll den Namen des Schülers bereits wissen
  filter(:full_name_search, :string, header: "Name") do |value|
     self.where(["clients.full_name_search = ? OR clients.last_name = ?",  "#{value.gsub(/\s*/,'')}", "#{value.strip}"])
  end

  # exakte suche nach vorname + name bzw. nur name, der Benutzer soll den Namen des Schülers bereits wissen
  filter(:guardian_name_search, :string, header: "Name der Eltern") do |value|
    self.includes(:guardians).where(["guardians.first_name ILIKE ? OR guardians.last_name ILIKE ?", "%#{value}%" , "%#{value}%" ])
  end

  filter(:portfolio_level, :enum, header: "Förderstufe", :select => ["0", "1", "2", "3"])



  column(:id, header: "ID",  order: :id, html: true) do |client|
    client.id
  end

  column(:former_client_status, header: "Status", html: true) do |client|
    if client.is_former_client && client.client_group_id.nil?
      r="ehemalig".html_safe
    end
    if client.is_former_client && client.client_group_id.present?
      r="abgehend".html_safe
    end
    unless client.is_former_client
      r="aktiv".html_safe
    end
    r
  end

  column(:name, header: "Name",  order: :last_name, html: true) do |client|
   show_client(client)
  end

  column(:gender, header: "Geschlecht", order: false, html: true) do |client|
    client.gender
  end

  column(:birthday, header: "Geburtsdatum", order: :bday)  do |c|
    format(c.bday) do |birthday|
      l(birthday, format: :short)
    end
  end

  column(:class, header: 'Klasse', html: true) do |c|
    show_client_group(c.client_group)
  end

  column(:institution, header: "Einrichtung", order: 'institutions.name')  do |c|
    format(c.institution) do |i|
      "#{link_to i.name, institution_path(i)}".html_safe
    end unless c.institution.blank?
  end

  column(:portfolio_level, header: "Förderstufe", order: false) do |c|
    format(c.portfolio_level) do |p|
      p
    end unless c.portfolio_level.blank?
  end

  column(:guardians, header: "Eltern", html: true) do |c|
    show_persons(c.guardians)
  end

  column(:actions,  header: '', :html => true) do |i|
    _wrp = 0
    update_button = ""
    delete_button = ""
      _wrp += 1
      read_button = "<a href='#{dashboard_client_path(i)}' role='button' class='btn btn-success'>
            <i class='glyphicon glyphicon-eye-open'></i>
          </a>"
    if can?(:update,i)
      _wrp += 1
      update_button = "<a href='#{edit_client_path(i)}' role='button' class='btn btn-success'>
        <i class='glyphicon glyphicon-pencil'></i>
      </a>"
    end
    if can?(:destroy,i)
      _wrp += 1
      delete_button = link_to  client_path(i), data: {:confirm => 'Sicher?'},
                               role: 'button', class: 'btn btn-danger', :method => :delete do
        content_tag(:i, nil, class: 'glyphicon glyphicon-remove')
      end
    end
    "<div class='btn-group btn-group-nowrap-#{_wrp} btn-group-xs  pull-right' >
 #{read_button}
    #{update_button}
    #{delete_button}
      </div>".html_safe

  end


end