module KaminariHelper

  # A helper that renders the pagination links.
  #
  #   <%= kaminari_paginate @articles %>
  #
  # ==== Options
  # * <tt>:window</tt> - The "inner window" size (4 by default).
  # * <tt>:outer_window</tt> - The "outer window" size (0 by default).
  # * <tt>:left</tt> - The "left outer window" size (0 by default).
  # * <tt>:right</tt> - The "right outer window" size (0 by default).
  # * <tt>:params</tt> - url_for parameters for the links (:controller, :action, etc.)
  # * <tt>:param_name</tt> - parameter name for page number in the links (:page by default)
  # * <tt>:remote</tt> - Ajax? (false by default)
  # * <tt>:ANY_OTHER_VALUES</tt> - Any other hash key & values would be directly passed into each tag as :locals value.
  def kaminari_paginate(scope, options = {}, &block)
    options[:total_pages] ||= options[:num_pages] || scope.total_pages
    options[:total_count] =  scope.total_count

    options[:window] = 4 unless options.has_key?(:window)
    options[:outer_window] = 0 unless options.has_key?(:outer_window)
    options[:left] = options[:outer_window] unless options.has_key?(:left)
    options[:right] = options[:outer_window] unless options.has_key?(:right)


    options = options.reverse_merge(:current_page => scope.current_page, :per_page => scope.limit_value, :remote => false)

    current_page_zi = options[:current_page] - 1 # indexed starting from 0
    total_pages_zi = options[:total_pages] - 1 # indexed starting from 0

    if(options[:window] - current_page_zi + options[:left] > 0 )
      options[:window] += (options[:window] - current_page_zi + options[:left]) # move otherwise missing buttons from right side to the left side
    end

    if(options[:window] + current_page_zi + options[:right] > total_pages_zi)
      options[:window] += (options[:window] + current_page_zi + options[:right] - total_pages_zi) # move otherwise missing buttons from left side to the right side
    end

    paginator = Kaminari::Helpers::Paginator.new(self, options)
    paginator.to_s
  end


end