class SendClientDuplicateRequestJob < ActiveJob::Base
  queue_as :default

  rescue_from ActiveJob::DeserializationError do |e|
    message = "\e[31m\e[1m#{e.message}\e[22m\e[0m" # make read and bold
    Rails.logger.error(message)
  end

  def perform(*client_duplicate_requests)
    client_duplicate_requests.each do |client_duplicate_request|
      # Create the client duplicate detection request at csb (if existing, this will raise a validation error)
      next if client_duplicate_request.request_retry? || client_duplicate_request.request_performed?
      begin
        client_duplicate_request.reset!
        CsbApi::ClientDuplicateDetectionRequest.create!({
           'client-duplicate-request-id' => client_duplicate_request.id,
           'vsb-id'  => SystemSetting.get_instance.vsb_id,
           'bday'  => client_duplicate_request.bday,
           'first-name'  => client_duplicate_request.first_name,
           'last-name'  => client_duplicate_request.last_name,
           'gender'  => client_duplicate_request.gender
        })
        set_success(client_duplicate_request)
      rescue JsonApiClient::Errors::ConnectionError, JsonApiClient::Errors::ServerError => e
        set_recoverable_error(client_duplicate_request,e.message)
      rescue StandardError,JsonApiClient::Errors::ValidationError => e
        set_fatal_error(client_duplicate_request,e.message)
      end
    end
  end

  private
  def set_success(client_duplicate_request)
    client_duplicate_request.request_performed!
    Rails.logger.debug("Successful send client_duplicate_request.id=#{client_duplicate_request.id} ")
  end
  def set_recoverable_error(client_duplicate_request,message)
    if client_duplicate_request.retry_count <= 2
      client_duplicate_request.increment! :retry_count
      retry_job wait: (client_duplicate_request.retry_count * client_duplicate_request.retry_count).seconds
      client_duplicate_request.request_retry!
      Rails.logger.info(message)
    else
      client_duplicate_request.request_failed!
      message = "\e[31m\e[1m#{message}\e[22m\e[0m" # make read and bold
      Rails.logger.error(message)
    end
  end
  def set_fatal_error(client_duplicate_request,message)
    client_duplicate_request.request_failed!
    message = "\e[31m\e[1m#{message}\e[22m\e[0m" # make read and bold
    Rails.logger.error(message)
  end



end
