module AddressGeocoder
  class Request
    def self.query(*q)
      self._get_query(q)
    end

    def self.search(string_or_query)
      Geocoder.search(string_or_query).map{|r| ResultWrapperFactory.createFromGeocoderResult(r)}
    end

    private
    def self._get_query(q)
      if q.length == 6
        return self._get_query_args(q[0],q[1],q[2],q[3],q[4],q[5])
      end
      if q.length == 1
        if q[0].is_a?(String)
          return  self._get_query_s(q[0])
        end
        if q[0].is_a?(Hash)
          return  self._get_query_h(q[0])
        end
        if q[0].is_a?(Array)
          return self._get_query_a(q[0])
        end
      end
      raise ArgumentError.new
    end

    def self._get_query_s(string)
      Geocoder::Query.new(string)
    end

    def self._get_query_h(opts)
      opts.transform_keys!(&:to_sym)
      self._get_query_args(opts[:street], opts[:number], opts[:city], opts[:region], opts[:zip], opts[:country])
    end

    def self._get_query_a(ary)
      self._get_query_args(ary[0], ary[1], ary[2], ary[3], ary[4], ary[5])
    end

    def self._get_query_args(street, number, city, region, zip, country)
      query = Geocoder::Query.new('')
      formatter =  QueryFormatterFactory.createFromGeocoderQuery(query)
      query.text = formatter.format(street, number, city, region, zip, country)
      query
    end

  end
end