class Checklist < ActiveRecord::Base


  #--- Associations
  belongs_to :portfolio_specialization        # can be NULL
  belongs_to :portfolio_item, :class_name => 'Portfolio::Item' # cannot be NULL

  belongs_to :teacher
  belongs_to :manager

  has_many :checklist_groups
  accepts_nested_attributes_for :checklist_groups, reject_if: :all_blank
  has_many :checklist_questions

  belongs_to :checklist_tpl
  validates :checklist_tpl, presence: true, on: :create

  # on create, the template tables are copied into the data tables.
  before_create do
    self.name = checklist_tpl.name
    self.checklist_type = checklist_tpl.checklist_type
    self.portfolio_specialization = checklist_tpl.portfolio_specialization
    checklist_tpl.checklist_group_tpls.each do |checklist_group_tpl|
      self.checklist_groups.build do |checklist_group|
        checklist_group.name = checklist_group_tpl.name
        checklist_group.is_mandatory = checklist_group_tpl.is_mandatory
        checklist_group.portfolio_specialization = checklist_group_tpl.portfolio_specialization
        checklist_group_tpl.checklist_question_tpls.each do |checklist_question_tpl|
          checklist_group.checklist_questions.build do |checklist_question|
            checklist_question.checklist = self
            checklist_question.name = checklist_question_tpl.name
            checklist_question.portfolio_specialization = checklist_question_tpl.portfolio_specialization
            checklist_question.is_checkable = checklist_question_tpl.is_checkable
            checklist_question.has_freetext = checklist_question_tpl.has_freetext
            checklist_question.has_portfolio_item_references = checklist_question_tpl.has_portfolio_item_references
            checklist_question.is_mandatory = checklist_question_tpl.is_mandatory
            checklist_question.accepts_upload = checklist_question_tpl.accepts_upload
          end
        end
      end
    end
  end


  def available_involved_teachers
    portfolio_item.available_involved_teachers
  end

end
