module UserMessageConcern
  extend ActiveSupport::Concern

  included do
    has_many :received_user_messages,->{where(message_type: UserMessage::TYPE_IN)}, as: :recipient, autosave: false, class_name: '::UserMessage'
    has_many :sent_user_messages,->{where(message_type: UserMessage::TYPE_OUT)}, as: :sender, autosave: false, class_name: '::UserMessage'

    has_many :received_user_messages_message_box,->{where(message_type: UserMessage::TYPE_IN, message_box: [UserMessage::BOX_NEW,UserMessage::BOX_READ])}, as: :recipient, autosave: false, class_name: '::UserMessage'

    has_many :received_user_messages_new,->{where(message_type: UserMessage::TYPE_IN, message_box: UserMessage::BOX_NEW)}, as: :recipient, autosave: false, class_name: '::UserMessage'
    has_many :received_user_messages_read,->{where(message_type: UserMessage::TYPE_IN, message_box: UserMessage::BOX_READ)}, as: :recipient, autosave: false, class_name: '::UserMessage'
    has_many :received_user_messages_deleted,->{where(message_type: UserMessage::TYPE_IN, message_box: UserMessage::BOX_DELETED)}, as: :recipient, autosave: false, class_name: '::UserMessage'

    has_many :sent_user_messages_message_box,->{where(message_type: UserMessage::TYPE_OUT, message_box: [UserMessage::BOX_NEW,UserMessage::BOX_READ])}, as: :sender, autosave: false, class_name: '::UserMessage'

    has_many :sent_user_messages_new,->{where(message_type: UserMessage::TYPE_OUT, message_box: UserMessage::BOX_NEW)}, as: :sender, autosave: false, class_name: '::UserMessage'
    has_many :sent_user_messages_read,->{where(message_type: UserMessage::TYPE_OUT, message_box: UserMessage::BOX_READ)}, as: :sender, autosave: false, class_name: '::UserMessage'
    has_many :sent_user_messages_deleted,->{where(message_type: UserMessage::TYPE_OUT, message_box: UserMessage::BOX_DELETED)}, as: :sender, autosave: false, class_name: '::UserMessage'

    def combined_user_messages_trash
      UserMessage.from("((#{sent_user_messages_deleted.to_sql}) UNION (#{received_user_messages_deleted.to_sql})) as user_messages")
    end


    def receive_message(sender,message_attributes = {})
      message_attributes[:recipient] = self
      message_attributes[:sender] = sender
      message_attributes[:message_type] = UserMessage::TYPE_IN
      message = UserMessage.new(message_attributes)
      message.save
      message
    end
    alias_method :write_message, :receive_message


    def receive_system_message(message_attributes = {})
      receive_message(nil,message_attributes)
    end
    alias_method :write_system_message, :receive_system_message
  end

end
