module Portfolio
  class Comment < ::ActiveRecord::Base
    self.table_name = 'portfolio_comments'
    self.inheritance_column = nil

    include ::AuthorConcern
    include ::SystemTaskReceivable
    include ::Publishable

    belongs_to :portfolio_item,
               foreign_key: :portfolio_item_id,
               class_name: '::Portfolio::Item',
               inverse_of: :portfolio_comments
    delegate :portfolio_subject, to: :portfolio_item

    belongs_to :teacher,
               foreign_key: :teacher_id,
               class_name: '::Teacher',
               inverse_of: :portfolio_comments

    mount_uploader :file, ::PortfolioCommentUploader


    def self.latest
      order(created_at: :desc)
    end

    def self.with_blocking_status
      where('item_blocked IS NOT NULL')
    end


    validates :portfolio_item_id, presence: true
    validates :comment, presence: true

    def can_block_item?(teacher_id)
      self.class.exists?(portfolio_item_id: portfolio_item_id, teacher_id: teacher_id, teacher_choose_reviewer: true)
    end

    def can_delegate?(teacher_id)
      self.class.exists?(portfolio_item_id: portfolio_item_id, teacher_id: teacher_id, teacher_choose_reviewer: true)
    end

    def teacher_name(teacher)
      name = teacher.first_name + ' ' + teacher.last_name
      add =[]
      if teacher.teacher_category.is_special_ed?
        add << 'Sonderpädagoge'
      end
      teacher.teacher_institution_relations.each do |rel|
        if rel.teacher_institution_category.is_management?
          add << "Leitung #{rel.institution.name}"
        end
      end
      if add.any?
        "#{name} (#{add.join(', ')})"
      else
        name
      end
    end

    def acceptable_teachers(user)
      case
        when user.person.is_a?(::Teacher)
          allowed_institution_ids = user.person.teacher_institution_relations.map do |i|
            i.institution_id
          end

          teachers = Teacher.joins(:teacher_category, :teacher_institution_relations)
              .where('teacher_institution_relations.institution_id': allowed_institution_ids)
              .where('teacher_categories.is_special_ed': true).all
        when user.person.is_a?(::Manager)
          allowed_institution_ids = user.person.institution.all_child_institutions.map do |i|
            i.child_id
          end

          teachers = Teacher.joins(teacher_institution_relations: :teacher_institution_category)
              .where('teacher_institution_relations.institution_id': allowed_institution_ids)
              .where('teacher_institution_categories.is_management': true).all
        else
          teachers = []
      end
      teachers
    end

  end
end


