module Portfolio
  class Dbrc < ::ActiveRecord::Base
    self.table_name = 'portfolio_dbrcs'
    self.inheritance_column = nil

    include ::SystemTaskReceivable
    include ::Publishable

    belongs_to :portfolio_item,
               foreign_key: :portfolio_item_id,
               class_name: '::Portfolio::Item',
               inverse_of: :portfolio_dbrcs
    delegate :portfolio_subject, to: :portfolio_item

    has_one :target_agreement,
            foreign_key: :portfolio_dbrc_id,
            class_name: '::Portfolio::ItemTarget',
            inverse_of:  :portfolio_dbrc,
            dependent: :destroy # refs #192
    accepts_nested_attributes_for :target_agreement, reject_if: Proc.new { |p| p['drc_item_id'].blank? }

    belongs_to :client,
               foreign_key: :client_id,
               class_name: '::Client',
               inverse_of: :portfolio_dbrcs

    belongs_to :questionnaire,
               foreign_key: :questionnaire_id,
               class_name: 'Questionnaire',
               inverse_of: :portfolio_dbrcs
    validates_presence_of :questionnaire_id

    belongs_to :questionnaire_result,
               foreign_key: :questionnaire_result_id,
               class_name: 'QuestionnaireResult',
               inverse_of: :portfolio_dbrcs
    validates_presence_of :questionnaire_result_id

    before_validation do
      if self.target_agreement && self.target_agreement.new_record?
        self.target_agreement.portfolio_item = portfolio_item
      end
      if self.target_agreement && self.target_agreement.new_record? && self.target_agreement.drc_item_id
        self.target_agreement.content =  drc_item_text(target_agreement.drc_item_id)
        if self.target_agreement.evaluation.present? && self.target_agreement.evaluation.portfolio_item_evaluation_dbr.characteristics.none?
          self.target_agreement.evaluation.portfolio_item_evaluation_dbr.characteristics.build(name: drc_item_text(target_agreement.drc_item_id), increase_is_positive: true)
        end
      end
      if self.target_agreement && self.target_agreement.new_record? &&  target_agreement.evaluation.present?
       self.target_agreement.evaluation.result_scheme = "::Portfolio::ItemTargetResult::Dbr"
      end
      true
    end


    before_create do
      self.client = portfolio_item.portfolio.client
      true
    end


    def drc_items
      if questionnaire.present? && questionnaire_result.present?
        return questionnaire_result.question.drc_items
      end
      return []
    end

    def questionnaire_options_for_select
      if portfolio_item && portfolio_item.portfolio.client
        return portfolio_item.portfolio.client.portfolio_items.map(&:questionnaires).flatten
                   .select{|q| q.type=='Questionnaires::ItrfTeacher'}
                   .select{|q| q.questionnaire_results.reject{|r| r.result.nil? }.any? }
                   .map {|q| ["#{q.questionnaire_name} vom #{I18n.l(q.portfolio_item.start,format: :short)}",q.id] }
      end
      []
    end

    def questionnaire_result_grouped_options_for_select
      _res ={}
      if questionnaire.present?
        questionnaire.questionnaire_results.reject{|r| r.result.blank? }.sort_by(&:result).reverse.group_by(&:result).each do |r,qs|
          qs.each do |q|
            _res[questionnaire.result_texts[r]] = [] unless _res[questionnaire.result_texts[r]]
            _res[questionnaire.result_texts[r]] << ["#{q.question.name}",q.id]
          end
        end
      end
      _res
    end

    def drc_item_options_for_select
      if questionnaire.present? && questionnaire_result.present?
        return questionnaire_result.question.drc_items.map{|di| [di.name,di.id]}
      end
      []
    end

    def drc_item_text(drc_item_id)
      questionnaire_result.question.drc_items.select{|di| di.id.to_s == drc_item_id.to_s}.first.try(&:name)
    end



    # Example starting on a Wednsday, 3 per day, without saturday. Every xx is template for datapoint
    # |MO|DI|MI|DO|FR|SA|SO|
    # |  |  |xx|xx|xx|  |  |
    # |  |  |xx|xx|xx|  |  |
    # |  |  |xx|xx|xx|  |  |
    #
    # |MO|DI|MI|DO|FR|SA|SO|
    # |xx|xx|  |  |  |  |  |
    # |xx|  |  |  |  |  |  |
    # |xx|  |  |  |  |  |  |
    #
    # example return:
    # [
    #     [# woche
    #         [0,0,1,1,1], #tr
    #         [0,0,1,1,1], #tr
    #         [0,0,1,1,1], #tr
    #     ],
    #     [# woche
    #         [1,1,0,0,0], #tr
    #         [1,0,0,0,0], #tr
    #         [1,0,0,0,0], #tr
    #     ]
    # ]
    def week_plan
      _weeks=[]
      _week=[]
      _day=[]
      _wday = (first_day_print.present? ? first_day_print.wday : Date.today.wday) # 0-6
      _empty_first_week = [0,_wday - 1].max
      _empty_first_week.times { _week << Array.new(data_points_per_day_print, false) }
      target_agreement.evaluation.portfolio_item_evaluation_dbr.number_of_data_points.times do
        _day << true
        if _day.size == data_points_per_day_print
          _week << _day
          _day = []
        end
        if _week.size == (saturday_print? ? 6 : 5)
          _weeks << _week
          _week = [ ]
        end
      end
      while _day.size < data_points_per_day_print
        _day << false
      end  if _day.any?
      _week << _day if _day.any?
      ((saturday_print? ? 6 : 5)-_week.size).times { _week << Array.new(data_points_per_day_print, false) }
      _weeks << _week if _week.any?
      _weeks.map(&:transpose) # transpose to ease iterating html table.
    end

  end
end


