module Portfolio
  class EvaluationInvolvedSupporter < ::ActiveRecord::Base
    self.table_name='evaluation_involved_supporters'
    self.inheritance_column = nil

    has_many :target_results,
             as: :evaluative,
             foreign_key: :evaluative_id,
             class_name: '::Portfolio::ItemTargetResult',
             dependent: :destroy # refs #192

    belongs_to :portfolio_item_evaluation,
               foreign_key: :portfolio_item_evaluation_id,
               class_name: '::Portfolio::ItemEvaluation',
               inverse_of: :evaluation_involved_supporters

    belongs_to :supporter,
               foreign_key: :supporter_id,
               class_name: '::Supporter'

    validates_uniqueness_of :portfolio_item_evaluation_id, :scope => :supporter_id

    # this has to be done when the result scheme of portfolio_item_evaluation changes.
    def handle_result_scheme_change!
      destroy_result!
      create_result!
    end

    # destroy all results for this person and this evaluation
    def destroy_result!
      portfolio_item_evaluation.portfolio_item_target.results.where( evaluative_id: supporter.id, evaluative_type: 'Supporter').destroy_all
    end

    # create a results for this person and this evaluation with the current result scheme
    def create_result!
      portfolio_item_evaluation.portfolio_item_target.results.create({result_scheme: portfolio_item_evaluation.result_scheme, evaluative: supporter})
    end

    before_save do
      unless portfolio_item_evaluation.portfolio_item_target.results.where( evaluative_id: supporter.id, evaluative_type: 'Supporter').any?
        create_result!
      end
      true
    end
  end
end


