module Portfolio
  class ItemClassDiagnostic < ::ActiveRecord::Base
      self.table_name = 'portfolio_item_class_diagnostics'
      self.inheritance_column = nil

      include ::Portfolio::Concerns::DiagnosticResultConcern
      self.allowed_involvements[:subject] = false
      self.allowed_involvements[:guardian] = false
      include ::Publishable
      include ::SystemTaskReceivable

      belongs_to :portfolio_item,
                 class_name: '::Portfolio::Item',
                 foreign_key: :portfolio_item_id,
                 inverse_of: :diagnostics
      delegate :portfolio_subject, to: :portfolio_item
      delegate :portfolio, to: :portfolio_item
      delegate :closed?, to: :portfolio_item

      belongs_to :diagnostic,
                 class_name: '::OnlineDiagnostic',
                 foreign_key: :diagnostic_id,
                 inverse_of: :portfolio_item_diagnostics
      accepts_nested_attributes_for :diagnostic, reject_if: :all_blank




      def remote_id=(remote_id)
        #abort ::CsbApi::OnlineDiagnostic.where(id: remote_id).includes('online-diagnostic-urls').first.inspect
        self.diagnostic=::CsbApi::OnlineDiagnostic.where(id: remote_id).includes('online-diagnostic-urls').first.try do |remote_diagnostic|
          remote_diagnostic.build_model(::OnlineDiagnostic.where(id: remote_id).first_or_initialize).tap do |local_diagnostic|
            local_diagnostic.save(validate: false)
          end
        end
      end

      def remote_id
        self.diagnostic.try(&:id)
      end

      def start
        self.begin
      end

      validates_presence_of :portfolio_item
      validates :begin, presence: true
      validates :begin, date: true
      validates :begin, chronological_order: {fields: ['portfolio_item.start',:begin]}
      validates :content, length: { minimum: 10 }




      # get the diagnostics that are allowed/availiable for the portfolio item
      #
      # the diagnostics are always fetched using the csb api. The returned diagnostics may or may not
      # exist locally.
      #
      # @return [Array<Diagnostic>]
      def available_remote_diagnostics
        ::CsbApi::OnlineDiagnostic.all.map(&:build_model)
      end

      #
      # @return [Array<Diagnostic>]
      def available_diagnostics
        ::OnlineDiagnostic.all
      end

    end
end

