module Portfolio
  class ItemInvolvedTeacher  < ::ActiveRecord::Base
      self.table_name = 'portfolio_item_involved_teachers'
      self.inheritance_column = nil

      belongs_to :portfolio_item ,
                 foreign_key: :portfolio_item_id,
                 class_name: '::Portfolio::Item',
                 inverse_of: :portfolio_item_involved_teachers
      delegate :portfolio_subject, to: :portfolio_item

      belongs_to :teacher,
                 foreign_key: :teacher_id,
                 class_name: '::Teacher',
                 inverse_of: :portfolio_item_involved_teachers

      validates_uniqueness_of :portfolio_item_id, :scope => :teacher_id
      validates :portfolio_item, :teacher, presence: true

      before_save :set_flags



      def set_flags
        return true if teacher.blank?
        set_class_teacher
        set_support_teacher
        set_special_ed
        set_management
        true
      end

      private

      def set_class_teacher
        self.class_teacher = portfolio_subject.is_class_teacher?(teacher)
      end

      def set_support_teacher
        self.support_teacher = portfolio_subject.is_support_teacher?(teacher)
      end

      def set_special_ed
        self.special_ed = portfolio_subject.is_special_ed?(teacher)
      end

      def set_management
        self.management = portfolio_subject.is_management?(teacher)
      end
    end
end

