module Portfolio
  class Rule < ::ActiveRecord::Base
    self.table_name = 'portfolio_rules'
    self.inheritance_column= nil

    has_many :portfolio_conditions,
             foreign_key: :rule_id,
             class_name: '::Portfolio::Condition',
             inverse_of: :portfolio_rule

    has_many :allowed_step_relations,
             foreign_key: :rule_id,
             class_name: '::Portfolio::RuleStep',
             inverse_of: :portfolio_rule


    has_many :allowed_portfolio_steps,
             class_name: '::Portfolio::Step' ,
             through: :allowed_step_relations,
             source: :portfolio_step

    has_many :template_documents,
             class_name: '::Portfolio::TemplateDocument' ,
             foreign_key: :rule_id,
             inverse_of: :portfolio_rule

    def description
      # refs  #405
      self[:description].html_safe
    end

    after_initialize :init
    def init
      @portfolio_match = false
      @all_notifiers = []
      return @all_notifiers if notifiers.nil?
      @all_notifiers = ::JSON.parse(notifiers).map do |object|
        instance = ('::Portfolio::Notifiers::'+object['type'].camelize).constantize.new
        object['args'].each do |k,v|
          instance.send(k+'=',v) if instance.respond_to? k+'='
        end unless object['args'].nil?
        instance.rule=self
        instance
      end
    end

    def all_notifiers(portfolio)
      @all_notifiers.map { |n| n.portfolio = portfolio; n; }
    end

    def allowed_portfolio_steps_for(enumerator)
      allowed_portfolio_steps.to_a.keep_if do |step|
        step.match_portfolio?(enumerator)
      end
    end

    def allow_level_raise(enumerator)
      portfolio_conditions.each do |c|
        if c.match_portfolio?(enumerator)
          # wenn eine bedingung gegen das portfolio matchen konnten, dann erlaube den stufenanstieg
          return true
        end
      end
      false
    end

    # @param enumerator [Portfolio::Enumerator]
    # @return [::Portfolio::Condition]
    def matching_condition(enumerator)
      portfolio_conditions.each do |c|
        if c.match_portfolio?(enumerator) && c.match_item?(enumerator.current_item)
          return c
        end
      end
      return nil
    end


  end
end
