module SystemTasks
  class ClientConfidentialityDocumentMissing < Base

    def _escalations
      [
          {
              Escalator::SendUserMessage => [target.author,target.editor]
          },
          {
              Escalator::SendUserMessage => [target.client.try(:teacher),target.client.try(:client_group).try(:teacher)],
              Escalator::SendEmail =>  [target.client.try(:teacher),target.client.try(:client_group).try(:teacher)]
          },
          {
              Escalator::SendUserMessage => target.client.institution.teacher_institution_relations.where(is_school_admin:true).all.to_a,
              Escalator::SendEmail =>  target.client.institution.teacher_institution_relations.where(is_school_admin:true).all.to_a
          },
          {
              Escalator::SendUserMessage => Manager.where(institution_id: InstitutionHierarchy.where(child_id: target.client.institution.id).select(:parent_id)).all.to_a,
              Escalator::SendEmail =>  Manager.where(institution_id: InstitutionHierarchy.where(child_id: target.client.institution.id).select(:parent_id)).all.to_a
          }
      ]
    end

    def is_manual_confirmable?
      true
    end

    class << self
      def subscribe_to
        [
            {
                klass: 'client_supporter_relation',
                callback: 'after_save',
                handler: 'client_supporter_relation_after_save'
            },
            {
                klass: 'document', # this is actually only a class handling documents for client_supporter_relations.
                callback: 'after_destroy',
                handler: 'document_after_destroy'
            }
        ]
      end

      def document_after_destroy(document)
        if Document.categories[document.category] == Document::ENUM_CATEGORY_CONFIDENTIALITY_DOCUMENT
          client_supporter_relation = document.attachable
          if client_supporter_relation.is_a? ClientSupporterRelation
            create({
                 target: client_supporter_relation,
                 client: client_supporter_relation.client,
                 institution: nil,
                 start_at: DateTime.now,
                 label: client_supporter_relation.client.full_name,
                 title:  "Schweigepflichtsentbindung für #{client_supporter_relation.client.first_name} #{client_supporter_relation.client.last_name} durch #{client_supporter_relation.supporter.first_name} #{client_supporter_relation.supporter.last_name} fehlt",
                 message:  ::I18n.translate('system_notifications.client_confidentiality_document_missing.content',
                                            client_name: client_supporter_relation.client.first_name + " " + client_supporter_relation.client.last_name,
                                            supporter_name: client_supporter_relation.supporter.first_name + " " + client_supporter_relation.supporter.last_name
                 ),
                 due_at: DateTime.now,
                 next_escalation: DateTime.now,
                 escalation_delay: 86400 * 7,
                 resolve_location: Rails.application.routes.url_helpers.edit_client_supporter_relation_path(client_supporter_relation)
             }) unless where(target: client_supporter_relation).any?
          end
        end
      end


      def client_supporter_relation_after_save(client_supporter_relation)
        unless client_supporter_relation.documents.where(category: Document::ENUM_CATEGORY_CONFIDENTIALITY_DOCUMENT).any?
            create({
                target: client_supporter_relation,
                client: client_supporter_relation.client,
                institution: nil,
                start_at: DateTime.now,
                label: client_supporter_relation.client.full_name,
                title:  "Schweigepflichtsentbindung für #{client_supporter_relation.client.first_name} #{client_supporter_relation.client.last_name} durch #{client_supporter_relation.supporter.first_name} #{client_supporter_relation.supporter.last_name} fehlt",
                message:  ::I18n.translate('system_notifications.client_confidentiality_document_missing.content',
                                           client_name: client_supporter_relation.client.first_name + " " + client_supporter_relation.client.last_name,
                                           supporter_name: client_supporter_relation.supporter.first_name + " " + client_supporter_relation.supporter.last_name
                ),
                due_at: DateTime.now,
                next_escalation: DateTime.now,
                escalation_delay: 86400 * 7,
                resolve_location: Rails.application.routes.url_helpers.edit_client_supporter_relation_path(client_supporter_relation)
            }) unless where(target: client_supporter_relation).any?
        else
          where(target: client_supporter_relation).each(&:destroy)
        end
      end
    end
  end
end