module SystemTasks
  class ClientExpertiseTeacher < Base


    def _escalations
      [
          {
              Escalator::SendUserMessage => [
                  target.expertise_manager,
                  target.teacher,
                  target.expertise_school_inspector,
                  target.client_group.try(&:teacher),
                  target.expertise_teacher
              ] + target.expertise_institution.teacher_institution_relations.all.to_a.select(&:is_management?).map(&:teacher),
              Escalator::SendUserMessage => [
                  target.expertise_teacher
              ]
          }
      ]
    end


    def is_manual_confirmable?
      true
    end

    class << self
      def subscribe_to
        [
            {
                klass: 'client',
                callback: 'after_save',
                handler: 'client_after_save'
            }
        ]
      end

      def client_after_save(client)
        if client.expertise_teacher_id_changed?
          where(target: client).delete_all
          if client.expertise_teacher_id.present?
            create({
                 target: client,
                 client: client,
                 institution: nil,
                 label: client.full_name,
                 title:  "Ein Sonderpädagoge wurde für das Gutachten für #{client.first_name} #{client.last_name} ausgewählt.",
                 message: "#{client.expertise_teacher.full_name} soll jetzt ein Gutachten durchführen und das Schülerportfolio erweitern. Im Anschluss muss im Rahmen des Gutachtens ein Ergebnis eingetragen werden. Das Gutachten kann alternativ auch durch den Gutachter abgewiesen werden.",
                 start_at: DateTime.now,
                 due_at: DateTime.now,
                 next_escalation: DateTime.now,
                 escalation_delay: 0,
                 resolve_location: Rails.application.routes.url_helpers.dashboard_client_path(client)
             })
          end
        end
      end
    end




  end
end