module SystemTasks
  class DiagnosticMeasurementResultIncomplete < Base

    def _escalations
      [
          {
              Escalator::SendUserMessage => [target.portfolio_item.author,target.portfolio_item.editor]
          },
          {
              Escalator::SendUserMessage => target.portfolio_item.involved_teachers.all.to_a,
              Escalator::SendEmail => [target.portfolio_item.author,target.portfolio_item.editor]
          }
      ]
    end

    def is_manual_confirmable?
      false
    end

    class << self
      def subscribe_to
        [
            {
                klass: 'portfolio/item_diagnostic_measurement',
                callback: 'before_save',
                handler: 'portfolio_item_diagnostic_measurement_before_save'
            },
            {
                klass: 'portfolio/item_diagnostic_measurement',
                callback: 'after_destroy',
                handler: 'portfolio_item_diagnostic_measurement_after_destroy'
            }
        ]
      end

      def portfolio_item_diagnostic_measurement_before_save(portfolio_item_diagnostic_measurement)
        if portfolio_item_diagnostic_measurement.results_closed
          where(target: portfolio_item_diagnostic_measurement).delete_all
        else
          create({
               target: portfolio_item_diagnostic_measurement,
               client: (portfolio_item_diagnostic_measurement.portfolio.subject.is_a? Client) ? portfolio_item_diagnostic_measurement.portfolio.subject : nil,
               institution: (portfolio_item_diagnostic_measurement.portfolio.subject.is_a? ClientGroup) ? portfolio_item_diagnostic_measurement.portfolio.subject_proxy.institution : nil,
               label: portfolio_item_diagnostic_measurement.portfolio.subject_proxy.name,
               title:  "Die Freie Diagnostik für #{portfolio_item_diagnostic_measurement.portfolio.subject_proxy.name} muss noch eingetragen werden.",
               message:  "Für die Massnahme #{portfolio_item_diagnostic_measurement.portfolio_item.portfolio_step.name}
                          vom #{::I18n.localize(portfolio_item_diagnostic_measurement.portfolio_item.start,format: :short)} fehlt noch die Auswertung.",
               start_at: portfolio_item_diagnostic_measurement.begin,
               due_at:  portfolio_item_diagnostic_measurement.begin,
               next_escalation:  portfolio_item_diagnostic_measurement.begin,
               escalation_delay: 86400 * 7,
               resolve_location: Rails.application.routes.url_helpers.edit_portfolio_item_path(portfolio_item_diagnostic_measurement.portfolio_item)
           })
        end
      end

      def portfolio_item_diagnostic_measurement_after_destroy(portfolio_item_diagnostic_measurement)
        where(target: portfolio_item_diagnostic_measurement).delete_all
      end

    end
  end
end