module SystemTasks
  class PortfolioCommentsUpdated < Base

    def _escalations
      [
          {
              Escalator::SendUserMessage => target.involved_teachers + target.evaluable_target_agreements.inject([]) do |memo,i|
                memo + i.evaluation.involved_teachers
              end + target.portfolio_comments.inject([]) do |memo,i|
                memo + [i.author,i.teacher]
              end,
              Escalator::SendEmail => target.involved_teachers + target.evaluable_target_agreements.inject([]) do |memo,i|
                memo + i.evaluation.involved_teachers
              end + target.portfolio_comments.inject([]) do |memo,i|
                memo + [i.author,i.teacher]
              end
          }
      ]
    end

    def is_manual_confirmable?
      true
    end

    class << self
      def subscribe_to
        [
            {
                klass: 'portfolio/comment',
                callback: 'after_create',
                handler: 'portfolio_comment_after_create'
            }

        ]
      end


      def portfolio_comment_after_create(portfolio_comment)
        where(target: portfolio_comment.portfolio_item).delete_all
        create({
             target: portfolio_comment.portfolio_item,
             client: (portfolio_comment.portfolio_item.portfolio.subject.is_a? Client) ? portfolio_comment.portfolio_item.portfolio.subject : nil,
             institution: (portfolio_comment.portfolio_item.portfolio.subject.is_a? ClientGroup) ? portfolio_comment.portfolio_item.portfolio.subject_proxy.institution : nil,
             start_at: DateTime.now,
             label: portfolio_comment.portfolio_item.portfolio.subject_proxy.name,
             title:  "Ein Portfolioeintrag wurde kommentiert von #{portfolio_comment.author.full_name}",
             message: "#{portfolio_comment.comment} (#{portfolio_comment.author.full_name})",
             due_at: DateTime.now,
             next_escalation: DateTime.now,
             escalation_delay: 86400,
             resolve_location: Rails.application.routes.url_helpers.portfolio_item_path(portfolio_comment.portfolio_item),
         })
      end

    end

  end
end