Rails.application.routes.draw do

  mount Ckeditor::Engine => '/ckeditor'

  resources :system_settings, only: [:new,:create] do
    collection do
      get :load_latest_portfolio_rules
      #get :load_updated_semesters
      get :show_change_semester
      patch :change_semester
    end
  end

  resources :system_tasks, only: [:destroy]


  resources :auth_roles do #, only:[:index,:edit,:update] do
    collection do
      get :all_defined_auth_items
      get :current_user_grants
      post :su_grants
    end
  end

  resources :supporters
  resources :guardians, only: [:destroy]# only: [:index, :destroy] # GUARDIAN LIST REMOVED FOR NOW
  resources :addresses, only: [] {collection {get :search}} # /addresses/search.json
  resources :institution_categories, except: [] {collection {post :load}}
  resources :teacher_categories
  resources :teacher_institution_categories
  resources :portfolio_specializations
  get 'configurations' => 'configuration#index'

  resources :portfolio_diagnostic_informal_results, only:[:new,:create,:edit,:update,:destroy]

  resources :notifications, only:[:index, :show, :destroy] do
    # SHOW converts the notification to flash and redirects to a location where the notification can be resolved.
    # DELETE:Some notifications do not require further actions and can be deleted directly with redirect back to previous location.
  end
  get 'notifications_:notificationable' => 'notifications#index', as: 'notifications_notificationable'

  # Password resend, create new account views incomplete
  devise_for :users, path_names: { registration: 'account' }, controllers: {
      registrations: 'registrations',
      sessions: 'sessions'
  }

  resources :client_groups, except: [:index] do # index is on institution
    collection do
      get :select_portfolio_item_date
      get :select_portfolio_item_step
    end
    get :select_institution, on: :new
  end
  resources :remote_diagnostics,  param: :catalog_id, only: [:index]
  resources :diagnostics,  param: :catalog_id do
    resources :diagnostic_borrows, only: [:new,:create,:edit,:update], shallow: true
    get :edit_stock, on: :member
    patch :update_stock, on: :member
    post :create_from_remote, on: :member
  end


  resources :remote_advancement_plans, param: :catalog_id, only: [:index]
  resources :advancement_plans,  param: :catalog_id do
    resources :advancement_plan_borrows, only: [:new,:create,:edit,:update], shallow: true
    get :edit_stock, on: :member
    patch :update_stock, on: :member
    post :create_from_remote, on: :member
  end

  #######################
  # this is a workaround: datagrid does not include query params when ordering, so we take the query param institution type and put it into path
  get '/institutions' => 'institutions#index_redirect'
  get '/institutions_t/*institution_type' => 'institutions#index'
  get '/institutions_t' => 'institutions#index'
  #######################
  resources :institutions, except: [:index] do
    resources :institution_documents, only: [:index,:create,:edit,:update,:show,:destroy], shallow: true
    resources :client_groups, only: [:index]
    resources :diagnostics, param: :catalog_id, only: [] do
      resources :diagnostic_borrows, only: [:index]
    end
    resources :advancement_plans, param: :catalog_id, only: [] do
      resources :advancement_plan_borrows, only: [:index]
    end
    get :select, on: :new
    delete 'logo' => 'institutions#destroy_logo', as: 'destroy_logo'
    get :preview_logo_pdf, defaults: { format: 'pdf' }
  end



  resources :teachers do
    resources :diagnostic_borrows, only: [:index]
    resources :advancement_plan_borrows, only: [:index]
    member do
      get :edit_roles
      get :reset_password
      post :update_roles
    end
  end

  resources :managers, except: [:show] do
    member do
      get :edit_roles
      get :reset_password
      post :update_roles
    end
  end


  resources :portfolio_items, only: [:show, :edit, :update, :create, :destroy , :new] do
    get :borrows, on: :member
    get :evaluations, on: :member
    post :close, on: :member
    post :discard, on: :member
    resources :portfolio_comments, shallow: true
  end
  resources :portfolio_item_evaluation_dbrs, only: [:update]
  resources :portfolio_item_targets, only: [:edit,:update] do# this is for results
    get :plan, on: :member
  end

  resources :questionnaire_results, only: [:index] do
    get :drc, on: :collection
  end
  resources :portfolio_diagnostics, only: [:edit,:update] # this is for results
  resources :portfolio_class_diagnostics, only: [:edit,:update] # this is for results
  resources :portfolio_diagnostic_measurements, only: [:edit,:update] # this is for results

  get 'dashboard' => 'user_pages#dashboard', as: 'user_dashboard'
  get 'user_messages' => 'user_pages#messages', as: 'user_messages'
  get 'user_messages_inbox' => 'user_pages#messages_inbox', as: 'messages_inbox'
  get 'user_messages_outbox' => 'user_pages#messages_outbox', as: 'messages_outbox'
  get 'user_messages_trash' => 'user_pages#messages_trash', as: 'messages_trash'
  get 'read_user_message' => 'user_pages#read_message', as: 'read_message'
  get 'trash_user_message' => 'user_pages#trash_message', as: 'trash_message'
  get 'restore_user_message' => 'user_pages#restore_message', as: 'restore_message'
  get 'delete_user_message' => 'user_pages#delete_message', as: 'delete_message'
  #get 'user_notifications_:notificationable' => 'user_pages#notifications', as: 'user_notifications_notificationable'


  resources :checklist_tpls

  resources :client_duplicate_requests, only:[ :show, :destroy ]
  resources :client_duplicate_responses, only: []# [:destroy]
  resources :portfolios, only: [:show]
  resources :clients do
    collection do
      get :select_portfolio_item_date
      get :select_portfolio_item_step
      get :select_new_client_institution
      get :edit_next_semester_client_groups
      post :update_next_semester_client_groups
      get :check_duplicates
    end
    member do
      get :edit_former_career
      post :update_former_career
      get :edit_career
      post :update_career
      get :dashboard
      post :raise_level
      post :lower_level
      get :new_from_client_duplicate_request # in this case, id is not client id but duplicate_request_id
      get :edit_expertise # feature-461
      post :update_expertise # feature-461
    end
    resources :career_items, except: [:show,:new,:create,:destroy]
    resources :career_items_client_group, except: [:show,:edit,:update,:destroy]
    resources :career_items_level, except: [:show,:edit,:update,:destroy]
    resources :career_items_teacher, except: [:show,:edit,:update,:destroy]
    resources :career_items_institution, except: [:show,:edit,:update,:destroy]
    resources :client_supporter_relations, except: [:index, :show], shallow: true do
      delete 'documents/:id' => 'client_supporter_relations#destroy_document', as: 'destroy_document'
    end
  end


  root 'user_pages#dashboard'

  namespace :vsb_api do
    namespace :v1 do
      # old duplicate detetcion strategy, <= vsb v0.15.0, csb v0.14.0
      #----------------------------------------------------------
      jsonapi_resources :client_duplicate_detection_requests, except: [:show,:edit,:destroy,:index,:update,:new]  # only create
      jsonapi_resources :client_duplicate_requests, except: [:edit,:destroy,:create,:new] # only update, :show, :index
      jsonapi_resources :client_duplicate_responses, except: [:show,:edit,:destroy,:index,:update,:new]  # only create
      #----------------------------------------------------------

      # new duplicate detetcion strategy, > vsb v0.15.0, csb v0.14.0
      #----------------------------------------------------------
      jsonapi_resources :similar_clients, except: [:show,:edit,:destroy,:update,:new,:create]  # only index
      #----------------------------------------------------------
    end
  end
end
