class AlterationsForEvaluationChanges < ActiveRecord::Migration
  def change

    create_table :portfolio_item_targets do |t|
      t.text :content, null: false  # mandatory description of the target agreement
      t.text :result_comment # optional text comment on the outcome of a target agreement
      t.boolean :is_successful, null: false, default: false # whether all portfolio_item_target_results are successful
      t.boolean :evaluate, null: false, default: false # if true, all involved persons must provide a portfolio_item_target_results record
      t.boolean :is_positive, null: false, default: true # how to interpret the portfolio_item_target_results (is_positive=true ^= a positive portfolio_item_target_result is a success)
      t.references :portfolio_item, index: {name: 'idx_pit_on_pi_id'}, foreign_key: true, null: false
      t.timestamps null: false
    end

    # additional persons that are added by the field 'comment on additional persons'
    create_table :evaluation_involved_persons do |t|
      t.string :name, null: false
      t.references :portfolio_item_evaluation, index: {name: 'idx_eip_on_pit_id'}, foreign_key: true, null: false
    end

    create_table :portfolio_item_target_results do |t|
      t.string :result_scheme, null: false, default: 'Portfolio::ItemTargetResult::Binary' # enumeration, the applied result scale
      t.integer :result_value, null: true # enumeration, the choosen value from the scale
      t.boolean :is_successful, null: false, default: false # whether this entry represents a success, based on the scheme and the is_positive value of the target
      t.text :result_comment # optional text comment on the outcome of a target agreement by a specific person
      t.references :portfolio_item_target, index: {name: 'idx_pitr_on_pit_id'}, foreign_key: true, null: false
      t.references :evaluative,  polymorphic: true, index: {name: 'idx_pitr_on_pers_id'}, null: false
      t.timestamps null: false
    end

    add_column :portfolio_items, :raise_portfolio_level, :boolean, null: false, default: false  # decision to raise portfolio level

    rename_column :portfolio_item_evaluations, :start, :end_date
    change_column_null(:portfolio_item_evaluations, :end_date, false)
    add_column :portfolio_item_evaluations, :start_date, :date
    add_column :portfolio_item_evaluations, :result_scheme, :string, null: false, default: 'Portfolio::ItemTargetResult::Binary'  # enumeration, the result scale that must be applied in all portfolio_item_target_results
    add_column :portfolio_item_evaluations, :is_successful, :boolean, null: false, default: false  # whether all portfolio_item_targets are successful

    rename_column :portfolio_item_evaluations, :result, :additions # text field with additions on all portfolio_item_targets

    remove_column :portfolio_item_evaluations, :is_positive, :boolean  # is moved to portfolio_item_targets
    remove_column :portfolio_item_evaluations, :time_span, :integer # is altered to end_date
  end
end
