class AlterInstitutionHierarchiesTrigger < ActiveRecord::Migration
  def change
    execute "
      CREATE OR REPLACE FUNCTION institution_hierarchies_update() RETURNS TRIGGER
      AS $institution_hierarchies_update$
          BEGIN
              IF (TG_OP = 'DELETE') THEN
                DELETE FROM institution_hierarchies WHERE child_id=OLD.id OR parent_id=OLD.id;
                UPDATE institutions SET parent_id=OLD.parent_id WHERE parent_id=OLD.id;
              ELSIF (TG_OP = 'UPDATE') THEN
                -- Rebuild the complete hierarchy, consider rebuilding partly when performance degrades
                DELETE FROM institution_hierarchies;
                WITH RECURSIVE institution_hierarchies_cte(child_id,parent_id) AS (
                  SELECT institutions.id, institutions.parent_id
                  FROM institutions WHERE  institutions.parent_id IS NOT NULL
                  UNION
                  SELECT institution_hierarchies_cte.child_id,  institutions.parent_id
                  FROM institution_hierarchies_cte JOIN institutions  ON institution_hierarchies_cte.parent_id = institutions.id
                  WHERE institutions.parent_id IS NOT NULL
                )  INSERT INTO institution_hierarchies (parent_id,child_id) SELECT institution_hierarchies_cte.parent_id,institution_hierarchies_cte.child_id FROM institution_hierarchies_cte;
              ELSIF (TG_OP = 'INSERT') THEN
                -- Rebuild the complete hierarchy, consider rebuilding partly when performance degrades
                DELETE FROM institution_hierarchies;
                WITH RECURSIVE institution_hierarchies_cte(child_id,parent_id) AS (
                  SELECT institutions.id, institutions.parent_id
                  FROM institutions WHERE  institutions.parent_id IS NOT NULL
                  UNION
                  SELECT institution_hierarchies_cte.child_id,  institutions.parent_id
                  FROM institution_hierarchies_cte JOIN institutions  ON institution_hierarchies_cte.parent_id = institutions.id
                  WHERE institutions.parent_id IS NOT NULL
                )  INSERT INTO institution_hierarchies (parent_id,child_id) SELECT institution_hierarchies_cte.parent_id,institution_hierarchies_cte.child_id FROM institution_hierarchies_cte;
              END IF;
              RETURN NULL;
          END;
      $institution_hierarchies_update$ LANGUAGE plpgsql;"

  end
end
